@@ -1,9 +1,8 @@
 /***************************************************************************/
 /*                             Ada Web Server                              */
-/*                                                                         */
-/*                        Copyright (C) 2000-2008                          */
-/*                                AdaCore                                  */
-/*                                                                         */
+/*
+--                     Copyright (C) 2000-2009, AdaCore                    --
+                                                                           */
 /* This library is free software; you can redistribute it and/or modify    */
 /* it under the terms of the GNU General Public License as published by    */
 /* the Free Software Foundation; either version 2 of the License, or (at   */
@@ -101,12 +100,24 @@ main (int argc, char *argv[])
 #endif
 
 #ifdef _WIN32
+
+  #if !defined SHUT_RD && defined SD_RECEIVE
+  #define SHUT_RD SD_RECEIVE
+  #endif
+  #if !defined SHUT_WR && defined SD_SEND
+  #define SHUT_WR SD_SEND
+  #endif
+  #if !defined SHUT_RDWR && defined SD_BOTH
+  #define SHUT_RDWR SD_BOTH
+  #endif
+
   #define ETIMEDOUT   WSAETIMEDOUT
   #define ENOTCONN    WSAENOTCONN
   #define EWOULDBLOCK WSAEWOULDBLOCK
   #define EINPROGRESS WSAEINPROGRESS
-  #define SHUT_RDWR   SD_BOTH
   #define EINTR       WSAEINTR
+  #define ENOBUFS     WSAENOBUFS
+  #define ENOMEM      WSAENOBUFS
   #define EAI_SYSTEM  EAI_AGAIN
   //  EAI_SYSTEM getaddrinfo error code does not supported by Win32, but need
   //  for correct error handling, we could use any Win32 socket error code
@@ -154,15 +165,18 @@ main (int argc, char *argv[])
 
   P ("\n--  AUTOMATICALLY GENERATED, DO NOT EDIT THIS FILE\n\n");
   P ("--  Generator: check_config\n\n");
+
   P ("with Interfaces.C.Strings;\n");
-  P ("with System;\n\n");
+  P ("with System;\n");
+#ifndef _WIN32
+  P ("with GNAT.OS_Lib;\n");
+#endif
+  P ("\n");
+
   P ("package AWS.OS_Lib is\n\n");
   P ("   use Interfaces;\n\n");
 
 #ifdef _WIN32
-  //  libaws_win32.a need for emulate some unix routines not available in
-  //  Win32.
-  P ("   pragma Linker_Options (\"-laws_win32\");\n");
   //  libws2_32.a need for getaddrinfo freeaddrinfo routines in Windows
   //  XP/2003.
   P ("   pragma Linker_Options (\"-lws2_32\");\n\n");
@@ -209,11 +223,15 @@ main (int argc, char *argv[])
   P ("   SOCK_STREAM  : constant := %d;\n", SOCK_STREAM);
   P ("   SOL_SOCKET   : constant := %d;\n", SOL_SOCKET);
   P ("   SHUT_RDWR    : constant := %d;\n", SHUT_RDWR);
+  P ("   SHUT_RD      : constant := %d;\n", SHUT_RD);
+  P ("   SHUT_WR      : constant := %d;\n", SHUT_WR);
   P ("   ETIMEDOUT    : constant := %d;\n", ETIMEDOUT);
   P ("   EWOULDBLOCK  : constant := %d;\n", EWOULDBLOCK);
   P ("   ENOTCONN     : constant := %d;\n", ENOTCONN);
   P ("   EINPROGRESS  : constant := %d;\n", EINPROGRESS);
   P ("   EINTR        : constant := %d;\n", EINTR);
+  P ("   ENOBUFS      : constant := %d;\n", ENOBUFS);
+  P ("   ENOMEM       : constant := %d;\n", ENOMEM);
   P ("   FIONBIO      : constant := %d;\n", FIONBIO);
   P ("   FIONREAD     : constant := %d;\n", FIONREAD);
   P ("   MSG_NOSIGNAL : constant := %d;\n\n", v_MSG_NOSIGNAL);
@@ -273,35 +291,71 @@ main (int argc, char *argv[])
   P ("   pragma Convention (C, Addr_Info);\n\n");
 
   P ("   function GetAddrInfo\n");
-  P ("     (node    : in C.Strings.chars_ptr;\n");
-  P ("      service : in C.Strings.chars_ptr;\n");
-  P ("      hints   : in Addr_Info;\n");
+  P ("     (node    : C.Strings.chars_ptr;\n");
+  P ("      service : C.Strings.chars_ptr;\n");
+  P ("      hints   : Addr_Info;\n");
   P ("      res     : not null access Addr_Info_Access) return C.int;\n\n");
 
-  P ("   procedure FreeAddrInfo (res : in Addr_Info_Access);\n\n");
+  P ("   procedure FreeAddrInfo (res : Addr_Info_Access);\n\n");
 
-  P ("   function GAI_StrError (ecode : in C.int)");
+  P ("   function GAI_StrError (ecode : C.int)");
+  P (" return C.Strings.chars_ptr;\n\n");
+
+  P ("   function Socket_StrError (ecode : Integer)");
   P (" return C.Strings.chars_ptr;\n\n");
 
   P ("   function Set_Sock_Opt\n");
-  P ("     (S       : in C.int;\n");
-  P ("      Level   : in C.int;\n");
-  P ("      OptName : in C.int;\n");
-  P ("      OptVal  : in System.Address;\n");
-  P ("      OptLen  : in C.int) return C.int;\n\n");
+  P ("     (S       : C.int;\n");
+  P ("      Level   : C.int;\n");
+  P ("      OptName : C.int;\n");
+  P ("      OptVal  : System.Address;\n");
+  P ("      OptLen  : C.int) return C.int;\n\n");
+
+  P ("   function C_Ioctl\n");
+  P ("     (S   : C.int;\n");
+  P ("      Req : C.int;\n");
+  P ("      Arg : access C.int) return C.int;\n\n");
+
+  P ("   function C_Close (Fd : C.int) return C.int;\n\n");
+
+  P ("   procedure WSA_Startup\n");
+  P ("     (Version : C.int; Data : System.Address)");
+#ifdef _WIN32
+  P (";\n");
+  P ("   pragma Import (Stdcall, WSA_Startup, \"WSAStartup\");\n\n");
+#else
+  P (" is null;\n");
+#endif
+
+  P ("   function Socket_Errno return Integer");
+#ifndef _WIN32
+  P (" renames GNAT.OS_Lib.Errno");
+#endif
+  P (";\n\n");
 
   P ("private\n\n");
 
   P ("   pragma Import (%s, GetAddrInfo, \"getaddrinfo\");\n", i_conv);
   P ("   pragma Import (%s, FreeAddrInfo, \"freeaddrinfo\");\n", i_conv);
 
+#ifdef _WIN32
   //  gai_strerror for Win32 inlined in WS2TCPIP.H and is not thread safe.
   //  We are using simple replacement in win32/gai_strerror.c
 
-#ifdef _WIN32
-  P ("   pragma Import (C, GAI_StrError, \"AWS_gai_strerror\");\n\n");
+  P ("   pragma Import (C, GAI_StrError, \"AWS_gai_strerror\");\n");
+
+  //  There is no way in Windows to get error message from socket error code.
+  //  Use the own partial implementation of the POSIX strerror.
+
+  P ("   pragma Import (C, Socket_StrError, \"socket_strerror\");\n");
+  P ("   pragma Import (Stdcall, C_Ioctl, \"ioctlsocket\");\n");
+  P ("   pragma Import (Stdcall, C_Close, \"closesocket\");\n");
+  P ("   pragma Import (Stdcall, Socket_Errno, \"WSAGetLastError\");\n");
 #else
-  P ("   pragma Import (C, GAI_StrError, \"gai_strerror\");\n\n");
+  P ("   pragma Import (C, GAI_StrError, \"gai_strerror\");\n");
+  P ("   pragma Import (C, Socket_StrError, \"strerror\");\n");
+  P ("   pragma Import (C, C_Ioctl, \"ioctl\");\n");
+  P ("   pragma Import (C, C_Close, \"close\");\n");
 #endif
 
   P ("   pragma Import (%s, Set_Sock_Opt, \"setsockopt\");\n\n", i_conv);
